/**
 * Copyright (c) 2021 Dominic Masters
 * 
 * This software is released under the MIT License.
 * https://opensource.org/licenses/MIT
 */

#pragma once
#include "../../libs.h"
#include "../../display/shader.h"
#include "../../display/primitive.h"
#include "../../display/primitives/skywall.h"
#include "../../vn/vnscene.h"
#include "../../vn/vncharacter.h"

#define POKER_WORLD_SEAT_DISTANCE -1
#define POKER_WORLD_SEAT_ROTATION(n) (n * mathDeg2Rad(45.0f))

#define POKER_WORLD_SEAT_POSITION_X(n) ( \
  POKER_WORLD_SEAT_DISTANCE * (float)sin(POKER_WORLD_SEAT_ROTATION(n)) \
)
#define POKER_WORLD_SEAT_POSITION_Y -0.2f
#define POKER_WORLD_SEAT_POSITION_Z(n) ( \
  POKER_WORLD_SEAT_DISTANCE * (float)cos(POKER_WORLD_SEAT_ROTATION(n)) \
)

typedef struct {
  primitive_t skywall;
} pokerworld_t;

/**
 * Initialize the poker renderer.
 * 
 * @param game Game to initialize for.
 */
void pokerWorldInit(pokergame_t *game);

/**
 * Adjusts the camera to look at a specific player. You can use this to look at
 * the dealer by referencing the POKER_PLAYER_COUNT as the index.
 * 
 * @param scene Scene to adjust.
 * @param playerIndex Player index to look at.
 */
void pokerWorldLookAtPlayer(vnscene_t *scene, uint8_t playerIndex);

/**
 * Render the poker world.
 * 
 * @param world Poker Game World.
 * @param engine Game engine.
 * @param assets Assets to use.
 */
void pokerWorldRender(
  pokerworld_t *world, engine_t *engine, pokergameassets_t *assets
);

/**
 * Cleanup the poker world.
 * @param game Game to clean up for.
 */
void pokerWorldDispose(pokergame_t *game);